#!/usr/bin/env python3
from __future__ import division
from __future__ import print_function
from glob import glob

chrom_files = glob("plink*GRCh38*map")
print(f"Found files: ")
for x in chrom_files:
  print(x)

if __name__ == "__main__":
  with open("genetic_map_GRCh38_merged.tab","wt") as out:
    header = "chrom pos recomb_rate pos_cm".split()
    print("\t".join(header),file=out)

    # The file is in PLINK map format.
    # Column 1 - chromosome code ("chr2")
    # Column 2 - variant identifier (they omitted this here)
    # Column 3 - genetic position (centimorgans)
    # Column 4 - physical position (basepairs)
    for f in chrom_files:
      print(f"Starting on: '{f}'")

      # Grab all lines at once to sort & later seek forward
      print(f"Reading lines from '{f}'...")
      lines = []
      with open(f) as fp:
        for line in fp:
          ls = line.split()
          ls[2] = float(ls[2])
          ls[3] = int(ls[3])
          lines.append(ls)

      # Sort on position
      lines = sorted(lines,key=lambda x: x[3])

      # Now estimate recombination rates and write out the data formatted
      # as the original recombination format expects:
      # Column 1 - chromosome code ("chr2")
      # Column 2 - physical position
      # Column 3 - recombination rate (cM/Mb)
      # Column 4 - genetic position (cM)
      print("Calculating recombination rates...")
      for i in range(len(lines)):
        line_i = lines[i]

        try:
          line_next = lines[i+1]
        except:
          # At the very last point of the chromosome, recombination rate is 0 by definition.
          recomb = 0.0
        else:
          cm_i = line_i[2]
          cm_next = line_next[2]
          pos_i = line_i[3]
          pos_next = line_next[3]
          recomb = (cm_next - cm_i) / ((pos_next - pos_i) / 1E6)

        chrom = str(line_i[0])
        if "chr" not in chrom:
          chrom = "chr" + chrom

        out_line = "\t".join([
          chrom,
          str(line_i[3]),
          "{:.04g}".format(recomb),
          str(line_i[2]),
        ])

        print(out_line,file=out)
